/*****************************************************************************
 * commands.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * Copyright (C) 2001 Chris Pinkham
 * Released under the terms of the GPL.
 * *NO WARRANTY*
 *
 * cpinkham@infi.net
 * http://www4.infi.net/~cpinkham/gyach/
 *****************************************************************************/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <time.h>

#include "support.h"

#include "config.h"

#ifdef USE_PTHREAD_CREATE
#include <pthread.h>
#endif

#include "callbacks.h"
#include "commands.h"
#include "interface.h"
#include "gyach.h"
#include "yahoochat.h"
#include "yahoolib.h"
#include "gyach_int.h"

#include "nix.h"
#include "main.h"

extern SESSION *GlobalSessionHook;

/* fixme: this is in main.c */
void append_to_textbox(GtkWidget * window, GtkWidget * textbox, char *text);
void append_to_textbox_color(GtkWidget * window, GtkWidget * textbox, char *text);
int chatOut( SESSION *sess, char *cmd );

char *find_whitespace( char *str ) {
	char *ptr = str;

	while(( *ptr ) && ( ! isspace( *ptr )))
		ptr++;

	return( ptr );
}

char *skip_whitespace( char *str ) {
	char *ptr = str;

	while(( *ptr ) && ( isspace( *ptr )))
		ptr++;

	return( ptr );
}

void cmd_join( char *room ) {
	/* format is "join roomname" */
	send_yahoo_text_command( "join ", room );
}

void cmd_goto( char *user ) {
    u_char  buf[4096];
    u_char  *ptr = create_yahoo_header( buf , PKT_OUT_GOTO );

    ptr = append_str( ptr , user );

    send_yahoo_packet( buf, ptr );
}

void cmd_follow( char *user ) {
	char buf[129] = "";
	int r;

	if ( *user ) {
		/* check to make sure we don't want to display who we're following */
		if ( strcmp( "?", user )) {
			/* make sure they don't try to follow themself */
			if ( !strcmp( user, GlobalSessionHook->user )) {
				sprintf( buf,
					"%sYou can't follow yourself!%s\n",
					YAHOO_COLOR_RED, YAHOO_COLOR_BLACK );
				append_to_textbox( chat_window, NULL, buf );
				return;
			}

			r = find_user_row( user );
			if ( r < 0 ) {
				sprintf( buf,
					"User %s%s%s is not in this room, can not follow.\n",
					YAHOO_COLOR_BLUE, user, YAHOO_COLOR_BLACK );
				append_to_textbox( chat_window, NULL, buf );
				return;
			}

			if ( follow_user ) {
				free( follow_user );
			}
			follow_user = strdup( user );
		}
	} else {
		if ( follow_user ) {
			free( follow_user );
		}
		follow_user = NULL;
	}

	if ( follow_user ) {
		sprintf( buf, "Following user: %s%s%s\n",
			YAHOO_COLOR_BLUE, follow_user, YAHOO_COLOR_BLACK );
	} else {
		sprintf( buf, "%sNot currently following anyone.%s\n",
			YAHOO_COLOR_RED, YAHOO_COLOR_BLACK );
	}
	append_to_textbox( chat_window, NULL, buf );
}

void cmd_invite( char *user ) {
    u_char  buf[4096];
    u_char  *ptr = create_yahoo_header( buf , PKT_OUT_INVITE );

    ptr = append_str( ptr , user );

    send_yahoo_packet( buf, ptr );
}

void cmd_cls() {
	ct_clear();
}

void cmd_tell( char *args ) {
	char *ptr;

	args = skip_whitespace( args );
	ptr = args;

	if ( strlen( ptr )) {
		/* they gave a user & message so send message */
		/* format is "tell dummytext\001user\001message" */
		ptr = find_whitespace( ptr );
		*ptr = '\001';

		send_yahoo_text_command( "tell tell\001", args );
	} else {
		/* no user or message supplied so open window */
		if ( ! pm_window ) {
			pm_window = create_pm_window();
			gtk_widget_show( pm_window );
		} else {
			append_to_textbox( chat_window, NULL,
				"***Unable to open PM window, already open.***\n" );
		}
	}
}

void cmd_away( char *msg ) {
	GtkWidget *tmp_widget;
	GtkWidget *cw = chat_window;
	char buf[31];
	int len;

	len = strlen( msg );

	if ( len == 0 ) {
		if (( custom_away_message ) &&
			( strlen( custom_away_message )))
			msg = custom_away_message;
		else
			msg = "Be Right Back";
		my_status = 10;
	} else if (( len == 1 ) &&
		( isdigit( msg[0] ))) {
		my_status = atoi( msg );
	} else if (( len == 2 ) &&
		( isdigit( msg[0] )) &&
		( isdigit( msg[1] ))) {
		my_status = atoi( msg );
	} else {
		my_status = 10;
	}

	if (( my_status <= 9 ) ||
		( my_status == 11 ) ||
		( my_status == 12 )) {
		sprintf( buf, "away away\300\200%d\300\200", my_status );
		send_yahoo_text_command( buf, "" );

		switch( my_status ) {
			case 0: tmp_widget = lookup_widget( cw, "status_here" );
					break;
			case 1: tmp_widget = lookup_widget( cw, "status_be_right_back" );
					break;
			case 2: tmp_widget = lookup_widget( cw, "status_busy" );
					break;
			case 3: tmp_widget = lookup_widget( cw, "status_not_at_home" );
					break;
			case 4: tmp_widget = lookup_widget( cw, "status_not_at_my_desk" );
					break;
			case 5: tmp_widget = lookup_widget( cw, "status_not_in_the_office");
					break;
			case 6: tmp_widget = lookup_widget( cw, "status_on_the_phone" );
					break;
			case 7: tmp_widget = lookup_widget( cw, "status_on_vacation" );
					break;
			case 8: tmp_widget = lookup_widget( cw, "status_out_to_lunch" );
					break;
			case 9: tmp_widget = lookup_widget( cw, "status_stepped_out" );
					break;
			case 11: tmp_widget = lookup_widget( cw, "status_autoaway" );
					break;
			case 12: tmp_widget = lookup_widget( cw, "status_invisible" );
					break;
			default: tmp_widget = lookup_widget( cw, "status_here" );
		}

		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(tmp_widget), 1 );
	} else {
		send_yahoo_text_command( "away away\300\200" "10\300\200", msg );

		tmp_widget = lookup_widget( cw, "status_custom" );
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(tmp_widget), 1 );
	}
}

void cmd_back( void ) {
	cmd_away( "0" );
}

void cmd_auto_away( char *value ) {
	char buf[129];

	if ( ! value[0] )
		return;

	auto_away= atoi( value );

	if ( ! auto_away)
		auto_away = 15;

	auto_away_time = time(NULL) + ( auto_away * 60 );

	write_config();

	sprintf( buf, "*** Auto-Away timer set to %d minute(s) ***\n", auto_away );
	append_to_textbox( chat_window, NULL, buf );
}

/* fixme, possibly use 0x42 the "think" code if can figure it out */
/* empty message generates random thought on yahoo server */
void cmd_think( char *msg ) {
	send_yahoo_text_command( "think ", msg );
}

void cmd_emote( char *msg ) {
    u_char  buf[4096];
    u_char  *ptr = create_yahoo_header( buf , PKT_OUT_EMOTE );

	ptr = append_str( ptr, "emote\001" );
	ptr = append_str( ptr, msg );

    send_yahoo_packet( buf, ptr );
}

void cmd_roll( char *arg ) {
	send_yahoo_text_command( "roll ", arg );
}

void cmd_connect() {
	on_connect_activate( NULL, NULL );
}

void cmd_disconnect() {
	on_disconnect_activate( NULL, NULL );
}

void cmd_reconnect() {
	on_disconnect_activate( NULL, NULL );
	login_to_yahoo_chat();
}

void cmd_help( char *arg ) {
	char buf[] = {
		"\n"
		"-------------------------------------------------------------------------\n"
		"Gyach v" VERSION
			" - Copyright 2001 Chris T. Pinkham, ALL RIGHTS RESERVED\n"
		"\n"
		"Chat Commands:\n"
		"  /autoaway [min]       set auto-away timer to 'min' minutes\n"
		"  /away [msg]           go away with custom auto-away message\n"
		"  /back                 come back from away status\n"
		"  /connect              open login window\n"
		"  /disconnect           disconnect from chat\n"
		"  /follow [user]        follow user\n"
		"  /follow               stop following\n"
		"  /friends              list yahoo buddy list\n"
		"  /friend [user]        add/remove user to buddy/friend list\n"
		"  /goto [user]          goto the user\n"
		"  /history              display command history\n"
		"  /ignore [user]        ignore/un-ignore user\n"
		"  /invite [user]        invite the user to the current room\n"
		"  /join [room]          join the room\n"
		"  /mute [user]          mute/un-mute user for this session\n"
		"  /online               print time online\n"
		"  /profile [user]       show user's profile\n"
		"  /quit\n"
		"  /reconnect            disconnect then auto-reconnect\n"
		"  /regex [regex]        add/remove regex to auto-ignore list\n"
		"  /reload               reload ignore and emote/alias lists\n"
		"  /tell [user] [text]   send PM to user\n"
		"  /think [text]         . o O( think text )\n"
		"  /unignore [user]      un-ignore user\n"
		"  /url [url]            paste url as a clickable link\n"
		"  /ver                  paste Gyach version (" VERSION ") and url\n"
		"  : [text]              send text as an emote\n"
		"  ! [command]           execute command and post results\n"
		"-------------------------------------------------------------------------\n"
		"\n" };

	ct_append_fixed( buf, strlen( buf ));
	ct_scroll_to_bottom();
}

/* fixme, fill in the rest of this function */
void cmd_friend( char *arg ) {
	char *ptr;
	char *end;
	char buf[128];

	ptr = skip_whitespace( arg );
	end = find_whitespace( ptr );

	while( *end ) {
		*end = '\0';
		if ( find_friend( ptr )) {
			remove_friend( ptr );
			sprintf( buf, "%sBuddy:%s %s%s removed as friend.\n",
				YAHOO_COLOR_BLUE, YAHOO_COLOR_RED, ptr, YAHOO_COLOR_BLACK );
			append_to_textbox_color( chat_window, NULL, buf );
		} else {
			add_friend( ptr );
			sprintf( buf, "%sBuddy:%s %s%s added as friend.\n",
				YAHOO_COLOR_BLUE, YAHOO_COLOR_RED, ptr, YAHOO_COLOR_BLACK );
			append_to_textbox_color( chat_window, NULL, buf );
		}

		ptr = end + 1;
		ptr = skip_whitespace( ptr );
		end = find_whitespace( ptr );
	}

	if ( find_friend( ptr )) {
		remove_friend( ptr );
		sprintf( buf, "%sBuddy:%s %s%s removed as friend.\n",
			YAHOO_COLOR_BLUE, YAHOO_COLOR_RED, ptr, YAHOO_COLOR_BLACK );
		append_to_textbox_color( chat_window, NULL, buf );
	} else {
		add_friend( ptr );
		sprintf( buf, "%sBuddy:%s %s%s added as friend.\n",
			YAHOO_COLOR_BLUE, YAHOO_COLOR_RED, ptr, YAHOO_COLOR_BLACK );
		append_to_textbox_color( chat_window, NULL, buf );
	}
}

void cmd_profile( char *user ) {
	char *ptr;
#ifdef USE_PTHREAD_CREATE
	pthread_t prof_thread;
#endif

	ptr = strdup( user );

#ifdef USE_PTHREAD_CREATE
	pthread_create( &prof_thread, NULL, fetch_profile, (void *)ptr );
#else
	fetch_profile( (void *)ptr );
#endif
}

void cmd_online( void ) {
	char buf[1025] = "";
	time_t current_time;
	int hours;
	int minutes;
	int seconds;

	if ( connect_time ) {
		current_time = time(NULL);

		hours = (int)(( current_time - connect_time ) / 3600);
		minutes = (int)(( current_time - connect_time ) % 3600 ) / 60;
		seconds = (int)(( current_time - connect_time ) % 60 );

		sprintf( buf, "*** online for %d:%02d:%02d ***\n",
			hours, minutes, seconds );
		append_to_textbox_color( chat_window, NULL, buf );
	} else {
		append_to_textbox_color( chat_window, NULL,
			"*** You don't appear to be online at the moment ***\n" );
	}
}

void cmd_url( char *url ) {
	char buf[1025] = "";
	time_t current_time;
	int hours;
	int minutes;

	if ( *url ) {
		strcat( buf, YAHOO_STYLE_URLON );
		strcat( buf, url );
		strcat( buf, YAHOO_STYLE_URLOFF );
		chatOut( GlobalSessionHook, buf );
	} else {
		current_time = time(NULL);

		hours = (int)(( current_time - connect_time ) / 3600);
		minutes = (int)(( current_time - connect_time ) % 3600 ) / 60;

		sprintf( buf, "%sGyach%s v%s %s%s%s online for %d:%02d",
			YAHOO_STYLE_BOLDON, YAHOO_STYLE_BOLDOFF, VERSION, 
			YAHOO_STYLE_URLON, GYACH_URL, YAHOO_STYLE_URLOFF,
			hours, minutes );
		chatOut( GlobalSessionHook, buf );
	}
}

void cmd_alias( char *args ) {
	char tmp_str[257];
	char *ptr;
	char *alias;
	char *def;
	int found;
	gchar *listitem[3];
	gchar tmp_item[12] = "tmp";
	gchar tmp_item2[12] = "tmp";
	gchar tmp_item3[12] = "tmp";

	listitem[0] = tmp_item;
	listitem[1] = tmp_item2;
	listitem[2] = tmp_item3;

	DBG( 11, "cmd_alias( '%s' )\n", args );

	strcpy( tmp_str, args );
	ptr = tmp_str;
	ptr = skip_whitespace( ptr );

	/* find alias string and null-terminate it */
	alias = ptr;
	ptr = find_whitespace( ptr );
	*ptr = '\0';

	/* find args string */
	ptr++;
	ptr = skip_whitespace( ptr );
	def = ptr;

	found = alias_find( alias );

	if ( found == -1 ) {
		/* new alias so add item*/
		found = gtk_clist_insert(GTK_CLIST(chat_emotions), 0, listitem );

		strcpy( tmp_item, alias );
		tmp_item[0] = toupper( tmp_item[0] );
		gtk_clist_set_text(GTK_CLIST(chat_emotions), found, 0, tmp_item );
	}

	/* now just set data for item */
	gtk_clist_set_text(GTK_CLIST(chat_emotions), found, 1, def );
	gtk_clist_set_text(GTK_CLIST(chat_emotions), found, 2, def );
	gtk_clist_sort(GTK_CLIST(chat_emotions));

	/* now save list */
	save_emotions();

	DBG( 12, "RETURN cmd_alias()\n" );
}

void cmd_unalias( char *args ) {
	char tmp_str[257];
	char *ptr;
	char *alias;
	int found;

	DBG( 11, "cmd_unalias( '%s' )\n", args );

	strcpy( tmp_str, args );
	ptr = tmp_str;
	ptr = skip_whitespace( ptr );

	/* find alias string and null-terminate it */
	alias = ptr;
	ptr = find_whitespace( ptr );
	*ptr = '\0';

	found = alias_find( alias );

	if ( found != -1 ) {
		/* delete the alias */
		gtk_clist_remove( GTK_CLIST(chat_emotions), found );

		/* now save list */
		save_emotions();
	}

	DBG( 12, "RETURN cmd_unalias()\n" );
}

void cmd_ignore( char *user ) {
	char *ptr;
	char *end;

	ptr = skip_whitespace( user );
	end = find_whitespace( ptr );

	while( *end ) {
		*end = '\0';
		ignore_toggle( ptr );

		ptr = end + 1;
		ptr = skip_whitespace( ptr );
		end = find_whitespace( ptr );
	}

	ignore_toggle( ptr );
}

void cmd_unignore( char *user ) {
	char *ptr;
	char *end;

	ptr = skip_whitespace( user );
	end = find_whitespace( ptr );

	while( *end ) {
		*end = '\0';

		if ( ignore_check( ptr ))
			ignore_toggle( ptr );

		ptr = end + 1;
		ptr = skip_whitespace( ptr );
		end = find_whitespace( ptr );
	}

	if ( ignore_check( ptr ))
		ignore_toggle( ptr );
}

void cmd_reload() {
	char buf[128];

	ignore_load();
	read_emotions();

	sprintf( buf, "*** Loaded files: %d aliases and %d ignored users ***\n",
		emotion_count, g_list_length( ignore_list ));

	append_to_textbox( chat_window, NULL, buf );
}

void cmd_mute( char *user ) {
	char *ptr;
	char *end;

	ptr = skip_whitespace( user );
	end = find_whitespace( ptr );

	while( *end ) {
		*end = '\0';
		mute_toggle( ptr );

		ptr = end + 1;
		ptr = skip_whitespace( ptr );
		end = find_whitespace( ptr );
	}

	mute_toggle( ptr );
}

void cmd_unmute( char *user ) {
	char *ptr;
	char *end;

	ptr = skip_whitespace( user );
	end = find_whitespace( ptr );

	while( *end ) {
		*end = '\0';

		if ( mute_check( ptr ))
			mute_toggle( ptr );

		ptr = end + 1;
		ptr = skip_whitespace( ptr );
		end = find_whitespace( ptr );
	}

	if ( mute_check( ptr ))
		mute_toggle( ptr );
}

void cmd_regex( char *regex ) {
	char *ptr;
	char *end;

	ptr = skip_whitespace( regex );
	end = find_whitespace( ptr );

	while( *end ) {
		*end = '\0';
		regex_toggle( ptr );

		ptr = end + 1;
		ptr = skip_whitespace( ptr );
		end = find_whitespace( ptr );
	}

	regex_toggle( ptr );
}

void cmd_packet( char *filename ) {
	FILE *fp;
	char line[81];
	char *ptr;
	char *data = NULL;
	int type;
	int flag;
	int size;
	int first = 1;
	int value = 0;
	
	fp = fopen( filename, "rt" );

	if ( ! fp ) {
		append_to_textbox( chat_window, NULL,
			"***Unable to open packet datafile!***\n" );
		return;
	}

	while( fgets( line, 80, fp )) {
		line[strlen(line)-1] = '\0';

		if ( strstr( line, "type =" )) {
			/* only process the first packet in the file for now */
			if ( ! first ) {
				/* process packet */
				if ( debug_packets )
					dump_packet();
				showYahooPacket( GlobalSessionHook );
			}

			first = 0;

			memset( &(GlobalSessionHook->yin), 0,
				sizeof( GlobalSessionHook->yin ));

			data = GlobalSessionHook->yin.data;

			sscanf( line, "type = %x, flag = %d, size = %d",
				&type, &flag, &size );

			GlobalSessionHook->yin.type = type;
			GlobalSessionHook->yin.flag = flag;
			GlobalSessionHook->yin.size = size;
		} else {
			ptr = line;
			while(( *ptr ) &&
				( *ptr != ' ' ) &&
				( *ptr != '\n' ) &&
				( *ptr != '\r' ) &&
				( *ptr != '#' ) &&
				( ptr != strstr( ptr, "  " ))) {
				sscanf( ptr, "%x ", &value );
				*(data++) = value;
				ptr += 3;
			}
			*data = '\0';
		}
	}
	fclose( fp );

	if ( debug_packets )
		dump_packet();
	showYahooPacket( GlobalSessionHook );
}

void cmd_quit() {
	if (io_callback_tag != 0) {
		gdk_input_remove( io_callback_tag );
	}

	closesocket(sess->sock); 
	sess->sock = INVALID_SOCKET; 
	sess->quit = 1; 

	gdk_threads_leave();

	gtk_main_quit();
}

void cmd_exec( char *cmd ) {
	char data[128];
	char buf[513] = "";
	char *ptr;
	char new_cmd[513];
	FILE *fp;

	if ( child_pid ) {
		append_to_textbox( chat_window, NULL,
			"***Unable to exec command, still waiting on last exec!***\n" );
		return;
	}

	sprintf( new_cmd, "%s 2>&1", cmd );

	child_pid = fork();

	if ( child_pid == 0 ) {
		/* we are in child */
		alarm( 3 );
		fp = popen( new_cmd, "r" );

		if ( fp ) {
			strcat( buf, "`" );
			strcat( buf, cmd );
			strcat( buf, "` ===>\n" );

			while(( strlen( buf ) < 512 ) &&
				  ( fgets( data, 512-strlen(buf), fp ))) {
				if (( strlen( buf ) + strlen( data )) < 512 )
					strcat( buf, data );
			}
			pclose( fp );

			ptr = buf + strlen( buf );
			ptr--;
			while(( *ptr == '\n' ) || ( *ptr == '\r' )) {
				*ptr = '\0';
				ptr--;
			}

			if ( strlen( buf ) == 511 ) {
				strcpy( buf+507, "...." );
			}

			send_yahoo_text_command( "say say\001", buf );
		}

		_exit(0);
	} else {
		child_timeout = time(NULL) + 4;
	}
}

int try_command( char *input ) {
	char *action = NULL;
	char *args = NULL;
	char tmp_input[ 512 ];

	strcpy( tmp_input, input );
	action = tmp_input + 1;

	action = skip_whitespace( action ); /* find head of command */
	args = action;
	args = find_whitespace( args ); /* find whitespace after command */
	if ( *args ) {
		*args = '\0';
		args++;
		args = skip_whitespace( args ); /* find text after whitespace */
	}

	if ( action[0] == '\0' )
		return( 0 );

	if ( !strcmp( "join", action )) {
		cmd_join( args );
	} else if ( !strcmp( "goto", action )) {
		cmd_goto( args );
	} else if (( !strcmp( "cls", action )) ||
			   ( !strcmp( "clear", action ))) {
		cmd_cls( args );
	} else if ( !strcmp( "profile", action )) {
		cmd_profile( args );
	} else if ( !strcmp( "follow", action )) {
		cmd_follow( args );
	} else if (( !strcmp( "call", action )) ||
			   ( !strcmp( "invite", action ))) {
		cmd_invite( args );
	} else if (( !strcmp( "tell", action )) ||
			   ( !strcmp( "pm", action ))) {
		cmd_tell( args );
	} else if ( !strcmp( "away", action )) {
		cmd_away( args );
	} else if ( !strcmp( "back", action )) {
		cmd_back();
	} else if ( !strcmp( "autoaway", action )) {
		cmd_auto_away( args );
	} else if ( !strcmp( "think", action )) {
		cmd_think( args );
	} else if ( !strcmp( "emote", action )) {
		cmd_emote( args );
	} else if ( !strcmp( "roll", action )) {
		cmd_roll( args );
	} else if ( !strcmp( "alias", action )) {
		cmd_alias( args );
	} else if ( !strcmp( "unalias", action )) {
		cmd_unalias( args );
	} else if ( !strcmp( "ignore", action )) {
		cmd_ignore( args );
	} else if ( !strcmp( "unignore", action )) {
		cmd_unignore( args );
	} else if ( !strcmp( "reload", action )) {
		cmd_reload();
	} else if ( !strcmp( "mute", action )) {
		cmd_mute( args );
	} else if ( !strcmp( "unmute", action )) {
		cmd_unmute( args );
	} else if ( !strcmp( "regex", action )) {
		cmd_regex( args );
	} else if ( !strcmp( "help", action )) {
		cmd_help( args );
	} else if ( !strcmp( "url", action )) {
		cmd_url( args );
	} else if ( !strcmp( "online", action )) {
		cmd_online();
	} else if ( !strcmp( "connect", action )) {
		cmd_connect();
	} else if ( !strcmp( "disconnect", action )) {
		cmd_disconnect();
	} else if ( !strcmp( "reconnect", action )) {
		cmd_reconnect();
	} else if (( !strcmp( "gyach", action )) ||
			   ( !strcmp( "ver", action ))) {
		cmd_url( "" );
	} else if (( !strcmp( "quit", action )) ||
			   ( !strcmp( "exit", action )) ||
			   ( !strcmp( "logoff", action )) ||
			   ( !strcmp( "logout", action ))) {
		cmd_quit();
	} else if ( !strcmp( "packet", action )) {
		cmd_packet( args );
	} else if ( !strcmp( "friends", action )) {
		show_friends();
	} else if ( !strcmp( "friend", action )) {
		cmd_friend( args );
	} else if ( !strcmp( "history", action )) {
		history_print();
	} else if (( !strcmp( "exec", action )) ||
			   ( !strcmp( "x", action ))) {
		cmd_exec( args );
	} else {
		return( 0 );
	}

	return( 1 );
}

